/***********************************************************************
 * Time-stamp: <Mittwoch, 09.05.2001, 14:31:36; edited by heiko>
 * 
 * Tina's random number generators
 *
 * testing random number generators 
 * tests jumping, splitting, storing and loading generator's status
 *
 * Copyright by Heiko Bauke, Magdeburg 2001 
 *
 * heiko.bauke@student.uni-magdeburg.de
 *
 * This is free software. You may use and distribute this software under
 * the terms of the GNU GENERAL PUBLIC LICENSE Version 2. See COPYING
 * for details.
 *
 ***********************************************************************/

#include <iostream>
#include <vector>
#include "trng.h"

using namespace TRNG;
using namespace std;


void plausibility_main(RNG &ra, RNG &rb) {
  long i, j, k, n;
  bool err, genok;
  vector<long> status;
  RNG *rc, *rd;
  genok=true;
  cout << "testing " << ra.name() << endl;
  // test jump
  n=1l;
  ra.reset();
  rb.reset();
  for (i=0l; i<20l; ++i) {
    ra.jump(i);
    for (j=0l; j<n; ++j)
      rb.rand();
    err=false;
    if (ra.rand()!=rb.rand())
      err=true;
    else
      if (ra.rand()!=rb.rand())
	err=true;
      else
	if (ra.rand()!=rb.rand())
	  err=true;
    if (err) {
      cout << ra.name() << ": error in Trng::jump" << endl;
      genok=false;
      break;
    }
    n<<=1;
  }
  // test split
  for (i=1l; i<=128l; ++i) {
    for (j=0l; j<i; j++) {
      ra.reset();
      rb.reset();
      for (k=0l; k<j; ++k)
	ra.rand();
      rb.split(i, j);
      err=false;
      if (ra.rand()!=rb.rand())
	err=true;
      else {
	for (k=0l; k<i-1l; ++k)
	  ra.rand();
	if (ra.rand()!=rb.rand())
	  err=true;
	else {
	  for (k=0l; k<i-1l; ++k)
	    ra.rand();
	  if (ra.rand()!=rb.rand())
	    err=true;
	}
      }
      if (err) {
	cout << ra.name() << ": error in Trng::split" << endl;
	genok=false;
	break;
      }
    }
  }
  // test savestatus / loadstatus 
  ra.reset();
  rb.reset();
  for (i=0l; i<271828l; ++i) {
    ra.rand();
    rb.rand();
  }
  ra.save_status(status);
  for (i=0l; i<314159l; ++i)
    ra.rand();
  ra.load_status(status);
  err=false;
  if (ra.rand()!=rb.rand())
    err=true;
  else
    if (ra.rand()!=rb.rand())
      err=true;
    else
      if (ra.rand()!=rb.rand())
	err=true;
  if (err) {
    cout << ra.name() << ": error in Trng::savestatus or Trng::loadstatus" << endl;
    genok=false;
  }
  // copy
  ra.reset();
  rc=ra.copy();
  rd=ra.copy();
  rc->split(2l, 0l);
  rd->split(2l, 1l);
  for (i=0l; i<16l; ++i)
    if (i%2l==0l) {
      if (ra.rand()!=rc->rand())
	genok=false;
    } else {
      if (ra.rand()!=rd->rand())
	genok=false;
    }
  cout << ra.name() << ": test ";
  if (genok)
    cout << "passed" << endl;
  else
    cout << "failed" << endl;
}

int main(void) {
  ParkMiller r1a, r1b;
  RAND r2a, r2b;
  LCG64 r3a, r3b;
  LMC2 r4a, r4b;
  LMC3 r5a, r5b;
  LMC4 r6a, r6b;
  CLCG2 r7a, r7b;
  CLCG3 r8a, r8b;
  CLCG4 r9a, r9b;
  EINV r10a, r10b;
  EINVLCG64 r11a, r11b;
  plausibility_main(r1a, r1b);
  plausibility_main(r2a, r2b);
  plausibility_main(r3a, r3b);
  plausibility_main(r4a, r4b);
  plausibility_main(r5a, r5b);
  plausibility_main(r6a, r6b);
  plausibility_main(r7a, r7b);
  plausibility_main(r8a, r8b);
  plausibility_main(r9a, r9b);
  plausibility_main(r10a, r10b);
  plausibility_main(r11a, r11b);
  return(EXIT_SUCCESS);
}



